SystemOrganization addCategory: #NewspeakSynchronization!
SystemOrganization addCategory: #'NewspeakSynchronization-Tests'!

Object subclass: #CanonicalRepresentation
	instanceVariableNames: 'isOrdered elements originalClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NewspeakSynchronization'!

!CanonicalRepresentation commentStamp: '<historical>' prior: 0!
Canonical representation for synchronization. Objects answer an instance of this class in respond to the message ''canonicalRepresentation'' when their internal data structures are not suitable for reflective analysis for the purpose of object synchronization.

This class was designed to be used by collections like Set and Dictionary, so a synchronization mechanism can examine the elements of such data structures without detailed knowledge of the data-structure implementation.

Notes:

Collection has 7 subclasses:

* Bag
	For the purposes of synchronization, a bag is just an object with a dictionary, and the bag itself need no special treatment.

* CharacterSet, and Matrix
	For the purposes of synchronization, just objects with an array which has a deterministic size.

* SequenceableCollection
	Some of its subclasses need special treatment, at least: Heap, OrderedCollection, Semaphore, and SourceFileArray.
	Two OrderedCollection may hold the same elements but the size of their array may vary or their indices may differ.

* Set
	Set and its subclass Dictionary both require special treatment. This should be in place.

* SkipList
	Further analysis needed.

* WeakRegistry
	Further analysis needed.

Some collections has blocks which are used for sorting them.

What is the canonical representation of a block (BlockContext)?

   Copyright 2008 Cadence Design Systems, Inc.
   
   Licensed under the Apache License, Version 2.0 (the ''License''); you may not use this file except in compliance with the License.  You may obtain a copy of the License at  http://www.apache.org/licenses/LICENSE-2.0
!

!CanonicalRepresentation class methodsFor: 'instance creation' stamp: 'ryan 6/13/2011 12:45'!
ordered: elements
	^self new isOrdered: true; elements: elements asArray! !

!CanonicalRepresentation class methodsFor: 'instance creation' stamp: 'ryan 6/13/2011 12:45'!
unordered: elements
	^self new isOrdered: false; elements: elements asArray! !

!CanonicalRepresentation methodsFor: 'as yet unclassified' stamp: 'Ahe 5/26/2008 16:43'!
= other
	self class == other class ifFalse: [^false].
	self isOrdered = other isOrdered ifFalse: [^false].
	self elements size = other elements size ifFalse: [^false].
	^isOrdered
		ifTrue: [
			self elements with: other elements do: [ :my :others |
				my == others ifFalse: [^false]].
			true]
		ifFalse: [self elements asIdentitySet = other elements asIdentitySet]! !

!CanonicalRepresentation methodsFor: 'accessing' stamp: 'ryan 6/13/2011 12:45'!
elements
	"Answer the value of elements"

	^ elements! !

!CanonicalRepresentation methodsFor: 'accessing' stamp: 'ryan 6/13/2011 12:45'!
elements: anObject
	"Set the value of elements"

	elements := anObject! !

!CanonicalRepresentation methodsFor: 'as yet unclassified' stamp: 'Ahe 5/27/2008 11:18'!
hash
	^elements inject: isOrdered hash into: [ :a :b | a bitXor: b identityHash]! !

!CanonicalRepresentation methodsFor: 'accessing' stamp: 'ryan 6/13/2011 12:45'!
isOrdered
	"Answer the value of isOrdered"

	^ isOrdered! !

!CanonicalRepresentation methodsFor: 'accessing' stamp: 'ryan 6/13/2011 12:45'!
isOrdered: anObject
	"Set the value of isOrdered"

	isOrdered := anObject! !

!CanonicalRepresentation methodsFor: 'accessing' stamp: 'ryan 6/13/2011 12:45'!
originalClass
	"Answer the value of originalClass"

	^ originalClass! !

!CanonicalRepresentation methodsFor: 'accessing' stamp: 'ryan 6/13/2011 12:45'!
originalClass: anObject
	"Set the value of originalClass"

	originalClass := anObject! !

!CanonicalRepresentation methodsFor: 'as yet unclassified' stamp: 'Ahe 5/28/2008 15:39'!
printOn: s
	super printOn: s.
	s nextPut: $(.
	originalClass ifNotNil: [
		s nextPutAll: originalClass name; nextPut: $:.
		elements isEmptyOrNil ifFalse: [s nextPut: $ ]].
	elements do: [ :ea | ea printOn: s] separatedBy: [s nextPutAll: '. '].
	s nextPut: $).! !

Object subclass: #TransactionID
	instanceVariableNames: 'uuid timeStamp authorFullName authorEmail'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NewspeakSynchronization'!

!TransactionID commentStamp: '<historical>' prior: 0!
Copyright 2008 Cadence Design Systems, Inc.
   
   Licensed under the Apache License, Version 2.0 (the ''License''); you may not use this file except in compliance with the License.  You may obtain a copy of the License at  http://www.apache.org/licenses/LICENSE-2.0
!

!TransactionID class methodsFor: 'as yet unclassified' stamp: 'Ahe 4/25/2008 15:41'!
authorFullName: authorFullName email: authorEmail
	^super new
		authorFullName: authorFullName;
		authorEmail: authorEmail! !

!TransactionID class methodsFor: 'as yet unclassified' stamp: 'ryan 6/13/2011 12:44'!
new
	self shouldNotImplement! !

!TransactionID methodsFor: 'as yet unclassified' stamp: 'Ahe 4/25/2008 16:03'!
<= other
	^timeStamp <= other timeStamp! !

!TransactionID methodsFor: 'as yet unclassified' stamp: 'ryan 6/13/2011 12:44'!
= other
	^self class == other class
		and: [uuid = other uuid
		and: [timeStamp = other timeStamp
		and: [authorFullName = other authorFullName
		and: [authorEmail = other authorEmail]]]]! !

!TransactionID methodsFor: 'accessing' stamp: 'ryan 6/13/2011 12:44'!
authorEmail
	"Answer the value of authorEmail"

	^ authorEmail! !

!TransactionID methodsFor: 'accessing' stamp: 'ryan 6/13/2011 12:44'!
authorEmail: anObject
	"Set the value of authorEmail"

	authorEmail := anObject! !

!TransactionID methodsFor: 'accessing' stamp: 'ryan 6/13/2011 12:44'!
authorFullName
	"Answer the value of authorFullName"

	^ authorFullName! !

!TransactionID methodsFor: 'accessing' stamp: 'ryan 6/13/2011 12:44'!
authorFullName: anObject
	"Set the value of authorFullName"

	authorFullName := anObject! !

!TransactionID methodsFor: 'as yet unclassified' stamp: 'Ahe 4/25/2008 15:50'!
hash
	^uuid hash! !

!TransactionID methodsFor: 'as yet unclassified' stamp: 'ryan 6/13/2011 12:44'!
initialize

	uuid:= UUID new.
	timeStamp:= DateAndTime now! !

!TransactionID methodsFor: 'as yet unclassified' stamp: 'ryan 6/13/2011 12:44'!
printOn: s "<WriteStream>"
	self class name printOn: s.
	s nextPut: $:.
	authorFullName printOn: s.
	s nextPut: $@.
	uuid printOn: s.! !

!TransactionID methodsFor: 'accessing' stamp: 'ryan 6/13/2011 12:44'!
timeStamp
	"Answer the value of timeStamp"

	^ timeStamp! !

!TransactionID methodsFor: 'accessing' stamp: 'ryan 6/13/2011 12:44'!
timeStamp: anObject
	"Set the value of timeStamp"

	timeStamp := anObject! !

!TransactionID methodsFor: 'accessing' stamp: 'ryan 6/13/2011 12:44'!
uuid
	"Answer the value of uuid"

	^ uuid! !

!TransactionID methodsFor: 'accessing' stamp: 'ryan 6/13/2011 12:44'!
uuid: anObject
	"Set the value of uuid"

	uuid := anObject! !

!Set class methodsFor: '*NewspeakSynchronization' stamp: 'Ahe 4/24/2008 21:46'!
fromCanonicalRepresentation: c
	| newInstance |
	newInstance := self new: c elements size.
	c elements do: [ :ea | newInstance add: ea].
	^newInstance! !

!Set methodsFor: '*NewspeakSynchronization' stamp: 'Ahe 4/24/2008 21:33'!
canonicalRepresentation
	^CanonicalRepresentation unordered: self asArray! !

!Dictionary class methodsFor: '*NewspeakSynchronization' stamp: 'Ahe 4/24/2008 21:47'!
fromCanonicalRepresentation: c
	| newInstance |
	newInstance := self new: c elements size.
	c elements do: [ :ea | newInstance at: ea key put: ea value].
	^newInstance! !

!Dictionary methodsFor: '*NewspeakSynchronization' stamp: 'Ahe 4/24/2008 21:32'!
canonicalRepresentation
	^CanonicalRepresentation unordered: self associations! !

TestCase subclass: #NewspeakSynchronizationTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NewspeakSynchronization-Tests'!

!NewspeakSynchronizationTestCase commentStamp: '<historical>' prior: 0!
Copyright 2008 Cadence Design Systems, Inc.
   
   Licensed under the Apache License, Version 2.0 (the ''License''); you may not use this file except in compliance with the License.  You may obtain a copy of the License at  http://www.apache.org/licenses/LICENSE-2.0
!

NewspeakSynchronizationTestCase subclass: #NewspeakSynchronizationCoreTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NewspeakSynchronization-Tests'!

!NewspeakSynchronizationCoreTest commentStamp: '<historical>' prior: 0!
Copyright 2008 Cadence Design Systems, Inc.
   
   Licensed under the Apache License, Version 2.0 (the ''License''); you may not use this file except in compliance with the License.  You may obtain a copy of the License at  http://www.apache.org/licenses/LICENSE-2.0
!

!NewspeakSynchronizationCoreTest methodsFor: 'as yet unclassified' stamp: 'ryan 6/13/2011 13:29'!
testSnapshot

	self assert: [self newspeakSynchronization snapshot size = self newspeakSynchronization snapshot asIdentitySet size]! !

!NewspeakSynchronizationCoreTest methodsFor: 'as yet unclassified' stamp: 'ryan 6/13/2011 13:30'!
testTransactionID
	| t1 t2 |

	t1:= TransactionID authorFullName: 'Chewbacca' email: 'chewie@MillenniumFalcon.com'.
	t2:= t1 copy.

	self assert: [t1 == t1].
	self deny: [t1 == t2].
	self assert: [t1 = t2].
	self assert: [t1 hash = t2 hash].

	t2:= t1 deepCopy.
	self deny: [t1 == t2].
	self assert: [t1 = t2].
	self assert: [t1 hash = t2 hash].

	t2 timeStamp: DateAndTime yesterday.
	self deny: [t1 = t2].

	self assert: [{t1. t2} sort = {t2. t1}].! !

!NewspeakSynchronizationTestCase class methodsFor: 'testing' stamp: 'Ahe 4/24/2008 18:59'!
isAbstract
	^name = #NewspeakSynchronizationTestCase! !

!NewspeakSynchronizationTestCase class methodsFor: 'accessing' stamp: 'Ahe 4/24/2008 18:59'!
resources
	^{MultiLanguageTestResource. NewspeakSynchronizationTestResource}! !

!NewspeakSynchronizationTestCase methodsFor: 'accessing' stamp: 'Ahe 4/24/2008 19:52'!
newspeakSynchronization
	^NewspeakSynchronizationTestResource current newspeakSynchronization! !

!NewspeakSynchronizationTestCase methodsFor: 'accessing' stamp: 'Ahe 4/24/2008 19:52'!
nof
	^NewspeakSynchronizationTestResource current nof! !

!NewspeakSynchronizationTestCase methodsFor: 'accessing' stamp: 'Ahe 4/24/2008 19:52'!
platform
	^NewspeakSynchronizationTestResource current platform! !

TestResource subclass: #NewspeakSynchronizationTestResource
	instanceVariableNames: 'platform vmMirror nof newspeakSynchronization'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NewspeakSynchronization-Tests'!

!NewspeakSynchronizationTestResource commentStamp: '<historical>' prior: 0!
Copyright 2008 Cadence Design Systems, Inc.
   
   Licensed under the Apache License, Version 2.0 (the ''License''); you may not use this file except in compliance with the License.  You may obtain a copy of the License at  http://www.apache.org/licenses/LICENSE-2.0
!

!NewspeakSynchronizationTestResource class methodsFor: 'as yet unclassified' stamp: 'bobw 8/21/2008 15:39'!
resources
	^{SystemSnapshotTestResource}! !

!NewspeakSynchronizationTestResource methodsFor: 'accessing' stamp: 'ryan 6/14/2011 09:33'!
newspeakSynchronization: anObject
	"Set the value of newspeakSynchronization"

	newspeakSynchronization := anObject! !

!NewspeakSynchronizationTestResource methodsFor: 'accessing' stamp: 'ryan 6/14/2011 09:33'!
nof
	"Answer the value of nof"

	^ nof! !

!NewspeakSynchronizationTestResource methodsFor: 'accessing' stamp: 'ryan 6/14/2011 09:33'!
nof: anObject
	"Set the value of nof"

	nof := anObject! !

!NewspeakSynchronizationTestResource methodsFor: 'accessing' stamp: 'ryan 6/14/2011 09:33'!
platform
	"Answer the value of platform"

	^ platform! !

!NewspeakSynchronizationTestResource methodsFor: 'accessing' stamp: 'ryan 6/14/2011 09:33'!
platform: anObject
	"Set the value of platform"

	platform := anObject! !

!NewspeakSynchronizationTestResource methodsFor: 'running' stamp: 'ryan 2/22/2012 12:14'!
setUp
	platform:= BlackMarket new.
	vmMirror:= SqueakVmMirror usingPlatform: platform.
	nof:= NewspeakObjectFormat vmMirror: vmMirror usingPlatform: platform.
	newspeakSynchronization:= NewspeakSynchronization usingPlatform: platform usingObjectFormat: nof withSystemObjects: self systemSnapshot! !

!NewspeakSynchronizationTestResource methodsFor: 'as yet unclassified' stamp: 'bobw 8/21/2008 15:40'!
systemSnapshot

	^SystemSnapshotTestResource current snapshot! !

!NewspeakSynchronizationTestResource methodsFor: 'running' stamp: 'ryan 6/14/2011 09:33'!
tearDown
	NoModificationError initialize.
	platform:= nil.
	nof:= nil.
	newspeakSynchronization:= nil! !

!NewspeakSynchronizationTestResource methodsFor: 'accessing' stamp: 'ryan 6/14/2011 09:33'!
vmMirror
	"Answer the value of vmMirror"

	^ vmMirror! !

!NewspeakSynchronizationTestResource methodsFor: 'accessing' stamp: 'ryan 6/14/2011 09:33'!
vmMirror: anObject
	"Set the value of vmMirror"

	vmMirror := anObject! !

TestResource subclass: #SystemSnapshotTestResource
	instanceVariableNames: 'snapshot'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NewspeakSynchronization-Tests'!

!SystemSnapshotTestResource methodsFor: 'as yet unclassified' stamp: 'ryan 2/22/2012 12:14'!
setUp

	super setUp.
	snapshot:= NewspeakSynchronization systemSnapshot: BlackMarket new.! !

!SystemSnapshotTestResource methodsFor: 'accessing' stamp: 'ryan 6/14/2011 09:32'!
snapshot
	"Answer the value of snapshot"

	^ snapshot! !

!SystemSnapshotTestResource methodsFor: 'accessing' stamp: 'ryan 6/14/2011 09:32'!
snapshot: anObject
	"Set the value of snapshot"

	snapshot := anObject! !

!ProtoObject class methodsFor: '*NewspeakSynchronization' stamp: 'Ahe 4/24/2008 21:43'!
fromCanonicalRepresentation: c
	^c! !

!ProtoObject methodsFor: '*NewspeakSynchronization' stamp: 'Ahe 4/24/2008 21:23'!
canonicalRepresentation
	^self! !
